/*
 * arch/arm/mach-shmobile/board-armadillo800eva.c
 *
 * Copyright (C) 2011 Atmark Techno, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <linux/platform_device.h>
#include <linux/delay.h>
#include <linux/io.h>
#include <linux/i2c.h>
#include <linux/dma-mapping.h>
#include <linux/serial_sci.h>
#include <linux/smsc911x.h>
#include <linux/gpio.h>
#include <linux/input.h>
#include <linux/input/rma1evb_btn.h>
#include <linux/mmc/host.h>
#include <linux/mmc/sh_mmcif.h>
#include <linux/mmc/sh_mobile_sdhi.h>
#include <linux/mfd/tmio.h>
#include <linux/spi/spi.h>
#include <linux/spi/sh_msiof.h>
#include <linux/usb/r8a66597.h>
#include <linux/usb/rmobile.h>
#include <media/sh_mobile_ceu.h>
#include <media/soc_camera.h>
#include <media/soc_camera_platform.h>

#include <video/sh_mobile_lcdc.h>
#include <sound/sh_fsi.h>

#include <mach/common.h>
#include <mach/sh7740.h>
#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/time.h>
#include <asm/hardware/cache-l2x0.h>

#include <linux/sh_eth.h>
#include <mach/rmobile_hdmi.h>
#include <mach/rmobile_sdenc.h>

static void __init gpio_no_direction(u32 addr)
{
	__raw_writeb(0x00, addr);
}

static struct sh_eth_plat_data sh_eth_platdata = {
	.phy = 0x00,
	.edmac_endian = EDMAC_LITTLE_ENDIAN,
	.register_type = SH_ETH_REG_GIGABIT,
	.phy_interface = PHY_INTERFACE_MODE_MII,
};

static struct resource sh_eth_resources[] = {
	{
		.start = 0xe9a00000,
		.end = 0xe9a00800 - 1,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = 0xe9a01800,
		.end = 0xe9a02000 - 1,
		.flags = IORESOURCE_MEM,
	},
	{
		.start = gic_spi(110),
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device eth_device = {
	.name = "sh-eth",
	.id = 0,
	.dev = {
		.platform_data = &sh_eth_platdata,
	},
	.resource = sh_eth_resources,
	.num_resources = ARRAY_SIZE(sh_eth_resources),
};

static void sdhi0_set_pwr(struct platform_device *pdev, int state)
{
	/* todo */
}

static irqreturn_t sdhi0_cd_irq(int irq, void *devid)
{
	struct mmc_host *mmc = devid;
	int val = gpio_get_value(GPIO_PORT167);

	if (val)
		set_irq_type(irq, IRQ_TYPE_EDGE_FALLING);
	else
		set_irq_type(irq, IRQ_TYPE_EDGE_RISING);

	if (!work_pending(&mmc->detect.work))
		mmc_detect_change(mmc, msecs_to_jiffies(100));

	return IRQ_HANDLED;
}

static int sdhi0_get_cd(struct platform_device *pdev)
{
	static int init = 0;
	struct mmc_host *mmc = platform_get_drvdata(pdev);
	int ret;
	int val = gpio_get_value(GPIO_PORT167);
	int irq = irqpin2irq(31);

	if (!init) {
		init = 1;
		ret = request_irq(irq, sdhi0_cd_irq, 0,
				  dev_name(&pdev->dev), mmc);
		if (val)
			set_irq_type(irq, IRQ_TYPE_EDGE_FALLING);
		else
			set_irq_type(irq, IRQ_TYPE_EDGE_RISING);
	}

	return !val;
}

static struct sh_mobile_sdhi_info sdhi0_info = {
	.dma_slave_tx	= SHDMA_SLAVE_SDHI0_TX,
	.dma_slave_rx	= SHDMA_SLAVE_SDHI0_RX,
	.tmio_caps	= MMC_CAP_SD_HIGHSPEED | MMC_CAP_SDIO_IRQ,
	.tmio_ocr_mask	= MMC_VDD_165_195 | MMC_VDD_32_33 | MMC_VDD_33_34,
	.set_pwr	= sdhi0_set_pwr,
	.get_cd		= sdhi0_get_cd,
};

static struct resource sdhi0_resources[] = {
	[0] = {
		.name	= "SDHI0",
		.start	= 0xe6850000,
		.end	= 0xe68500ff,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(117),
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.start	= gic_spi(118),
		.flags	= IORESOURCE_IRQ,
	},
	[3] = {
		.start	= gic_spi(119),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device sdhi0_device = {
	.name		= "sh_mobile_sdhi",
	.id		= 0,
	.dev		= {
		.platform_data	= &sdhi0_info,
	},
	.num_resources	= ARRAY_SIZE(sdhi0_resources),
	.resource	= sdhi0_resources,
};

static void sdhi1_set_pwr(struct platform_device *pdev, int state)
{
}

static struct sh_mobile_sdhi_info sdhi1_info = {
	.dma_slave_tx	= SHDMA_SLAVE_SDHI1_TX,
	.dma_slave_rx	= SHDMA_SLAVE_SDHI1_RX,
	.tmio_caps	= MMC_CAP_SD_HIGHSPEED | MMC_CAP_SDIO_IRQ,
	.tmio_ocr_mask	= MMC_VDD_165_195 | MMC_VDD_32_33 | MMC_VDD_33_34,
	.set_pwr	= sdhi1_set_pwr,
};

static struct resource sdhi1_resources[] = {
	[0] = {
		.name	= "SDHI1",
		.start	= 0xe6860000,
		.end	= 0xe68600ff,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(121),
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.start	= gic_spi(122),
		.flags	= IORESOURCE_IRQ,
	},
	[3] = {
		.start	= gic_spi(123),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device sdhi1_device = {
	.name		= "sh_mobile_sdhi",
	.id		= 1,
	.dev		= {
		.platform_data	= &sdhi1_info,
	},
	.num_resources	= ARRAY_SIZE(sdhi1_resources),
	.resource	= sdhi1_resources,
};

static struct resource sh_mmcif_resources[] = {
	[0] = {
		.name	= "MMCIF",
		.start	= 0xe6bd0000,
		.end	= 0xe6bd00ff,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		/* MMC ERR */
		.start	= gic_spi(56),
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		/* MMC NOR */
		.start	= gic_spi(57),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct sh_mmcif_dma sh_mmcif_dma = {
	.chan_priv_rx	= {
		.slave_id	= SHDMA_SLAVE_MMCIF_RX,
	},
	.chan_priv_tx	= {
		.slave_id	= SHDMA_SLAVE_MMCIF_TX,
	},
};

static void sh_mmcif_set_pwr(struct platform_device *pdev, int state)
{
}

static struct sh_mmcif_plat_data sh_mmcif_plat = {
	.sup_pclk	= 0,
	.ocr		= MMC_VDD_165_195 | MMC_VDD_32_33 | MMC_VDD_33_34,
	.caps		= MMC_CAP_4_BIT_DATA | MMC_CAP_8_BIT_DATA |
			  MMC_CAP_NONREMOVABLE,
	.dma		= &sh_mmcif_dma,
	.set_pwr	= sh_mmcif_set_pwr,
};

static struct platform_device sh_mmcif_device = {
	.name		= "sh_mmcif",
	.id		= 0,
	.dev		= {
		.platform_data	= &sh_mmcif_plat,
	},
	.num_resources	= ARRAY_SIZE(sh_mmcif_resources),
	.resource	= sh_mmcif_resources,
};

static struct sh_fsi_dma sh_fsi2_dma = {
	.dma_porta_tx = {
		.slave_id	= SHDMA_SLAVE_FSIA_TX,
	},
	.dma_porta_rx = {
		.slave_id	= SHDMA_SLAVE_FSIA_RX,
	},
	.dma_portb_tx = {
		.slave_id	= SHDMA_SLAVE_FSIB_TX,
	},
};

static int fsi_set_rate(int is_porta, int rate)
{
	u32 *reg;
	int ret;

	/* set_rate is not needed if port A */
	if (is_porta)
		return 0;

	__raw_writeb(0x02, 0xe605000b);

	/* FSIBCKCR */
	reg = ioremap_nocache(0xe6150090, 4);
	if (reg != NULL) {
		*reg = 0x00000040;
		iounmap(reg);
	}

	/* FSIDIVB */
	reg = ioremap_nocache(0xfe1f8008, 4);
	if (reg != NULL) {
		*reg = 0x00000001;
		iounmap(reg);
	}

	switch (rate) {
	case 48000:
		reg = ioremap_nocache(0xfe1f0324, 4);
		if (reg != NULL) {
			*reg = 0x00000040;
			iounmap(reg);
		}
		ret = SH_FSI_ACKMD_256 | SH_FSI_BPFMD_64;
		break;
	default:
		pr_err("unsupported rate in FSI2 port B\n");
		ret = -EINVAL;
		break;
	}
	return ret;
}

static struct sh_fsi_platform_info fsi_info = {
	.porta_flags = SH_FSI_OUT_SLAVE_MODE	|
		       SH_FSI_IN_SLAVE_MODE	|
		       SH_FSI_BRS_INV		|
		       SH_FSI_OFMT(I2S)		|
		       SH_FSI_IFMT(I2S),
	.portb_flags = SH_FSI_BRS_INV		|
		       SH_FSI_BRM_INV		|
		       SH_FSI_LRS_INV		|
		       SH_FSI_OFMT(SPDIF),
	.portb_flags2 =
		       SH_FSI_LRCLK_256	|
		       SH_FSI_BITCLK_64,
	.set_rate = fsi_set_rate,
	.dma = &sh_fsi2_dma,
};

static struct resource fsi_resources[] = {
	[0] = {
		.name	= "FSI",
		.start	= 0xfe1f0000,
		.end = 0xfe1f8400 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start  = gic_spi(9),
		.flags  = IORESOURCE_IRQ,
	},
};

static struct platform_device fsi_device = {
	.name		= "sh_fsi2",
	.id		= 0,
	.num_resources	= ARRAY_SIZE(fsi_resources),
	.resource	= fsi_resources,
	.dev	= {
		.platform_data	= &fsi_info,
	},
};

static struct platform_device fsi_hdmi_codec_device = {
	.name = "fsi_hdmi_codec",
	.id = 0,
};

static struct sh_mobile_hdmi_info hdmi_info = {
	.source_dev_name = "sh_mobile_lcdc_fb.1",
};

static struct resource hdmi_resources[] = {
	[0] = {
		.start	= 0xe6be0000,
		.end = 0xe6be8000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start = gic_spi(131),
		.flags = IORESOURCE_IRQ,
	},
};

static struct platform_device hdmi_device = {
	.name = "sh_mobile_hdmi",
	.id = 0,
	.num_resources = ARRAY_SIZE(hdmi_resources),
	.resource = hdmi_resources,
	.dev = {
		.platform_data = &hdmi_info,
	},
};

static struct sh_mobile_sdenc_info sdenc_info = {
	.source_dev_name = "sh_mobile_lcdc_fb.1",
};

static struct resource sdenc_resources[] = {
	[0] = {
		.start	= 0xffcb0000,
		.end = 0xffcb1000 - 1,
		.flags	= IORESOURCE_MEM,
	},
};

static struct platform_device sdenc_device = {
	.name = "sh_mobile_sdenc",
	.id = 0,
	.num_resources = ARRAY_SIZE(sdenc_resources),
	.resource = sdenc_resources,
	.dev = {
		.platform_data = &sdenc_info,
	},
};

#define USBCR1		0xe605810a
#define USBH_USBCTR	0xf6710834
static struct clk *usb_clk, *usbp_clk, *usbh_clk, *usbphy_clk;
static DEFINE_SPINLOCK(usb_lock);
static void usb_common_clock(int on)
{
	static int count = 0;
	unsigned long flags;

	spin_lock_irqsave(&usb_lock, flags);
	if (on && !count) {
		clk_enable(usb_clk);
		clk_enable(usbp_clk);
		clk_enable(usbh_clk);
		clk_enable(usbphy_clk);

		if (gpio_get_value(GPIO_PORT159)) {
			/* Port1 and Port2 are HOST */
			__raw_writew(0x6b50, USBCR1);
		} else {
			/* Port1:Function Port2:HOST
			 * USBCR.P1PORTSEL = 11b(Port1 is driven by UFB FUNCTION)
			 * USBCR.P2PORTSEL = 01b(Port2 is driven by USB HOST)
			 * USBCR.PORTNUM   = 01b(One HOST)
			 */
			__raw_writew(0xd750, USBCR1);
		}

		mdelay(1);
		__raw_writel(0x0000040c, USBH_USBCTR);
		__raw_writel(0x00000408, USBH_USBCTR);
	}

	count = on ? count+1 : count-1;
	if (count < 0) {
		printk(KERN_ERR "%s: count error\n", __func__);
		count = 0;
		spin_unlock_irqrestore(&usb_lock, flags);
		return;
	}

	if (!on && !count) {
		__raw_writel(0x0000010f, USBH_USBCTR);
		__raw_writew(0x6bc0, USBCR1);
		clk_disable(usbphy_clk);
		clk_disable(usbh_clk);
		clk_disable(usbp_clk);
		clk_disable(usb_clk);
	}
	spin_unlock_irqrestore(&usb_lock, flags);
}

static struct rmobile_usb_platdata usb_pdata = {
	.common_clock	= usb_common_clock,
};

static u64 usb_dmamask = DMA_BIT_MASK(32);

static struct resource ohci_resources[] = {
	[0] = {
		.start	= 0xc6700000,
		.end	= 0xc6720000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(234),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device ohci_device = {
	.name	= "rmobile-ohci-driver",
	.id	= 0,
	.dev	= {
		.dma_mask		= &usb_dmamask,
		.coherent_dma_mask	= DMA_BIT_MASK(32),
		.platform_data		= &usb_pdata,
	},
	.num_resources	= ARRAY_SIZE(ohci_resources),
	.resource	= ohci_resources,
};

static struct resource ehci_resources[] = {
	[0] = {
		.start	= 0xc6700000,
		.end	= 0xc6720000 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(234),
		.flags	= IORESOURCE_IRQ,
	},
};
static struct platform_device ehci_device = {
	.name	= "rmobile-ehci-driver",
	.id	= 0,
	.dev	= {
		.dma_mask		= &usb_dmamask,
		.coherent_dma_mask	= DMA_BIT_MASK(32),
		.platform_data		= &usb_pdata,
	},
	.num_resources	= ARRAY_SIZE(ehci_resources),
	.resource	= ehci_resources,
};

static struct r8a66597_platdata usb_func_data = {
	.on_chip	= 1,
	.buswait	= 5,
	.vbus_irq	= irqpin2irq(7),
	.vbus_gpio	= GPIO_PORT209,
	.common_clock	= usb_common_clock,
};

static struct resource usb_func_resources[] = {
	[0] = {
		.name	= "USBHS",
		.start	= 0xe6890000,
		.end	= 0xe6890102 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(51),
		.flags	= IORESOURCE_IRQ,
	},
	[2] = {
		.name	= "USBHS-DMA",
		.start	= 0xe68a0000,
		.end	= 0xe68a0064 - 1,
		.flags	= IORESOURCE_MEM,
	},
	[3] = {
		.start	= gic_spi(49),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device __maybe_unused usb_func_device = {
	.name	= "r8a66597_udc",
	.id	= 0,
	.dev = {
		.dma_mask		= NULL,         /* not use dma */
		.coherent_dma_mask	= DMA_BIT_MASK(32),
		.platform_data		= &usb_func_data,
	},
	.num_resources	= ARRAY_SIZE(usb_func_resources),
	.resource	= usb_func_resources,
};

#include <linux/pwm_backlight.h>
#include "tpu-pwm.h"
static struct resource pwm_resources[] = {
	[0] = {
		.start = 0xe6600000,
		.end = 0xe66000ff,
		.flags = IORESOURCE_MEM,
	},
};

static struct delayed_work pwm_work;

static void pwm_real_init(struct work_struct *work)
{
	gpio_request(GPIO_FN_PORT202_TPU0TO2, NULL);
}

static void pwm_init(struct pwm_device *pwm)
{
	INIT_DELAYED_WORK(&pwm_work, pwm_real_init);
	schedule_delayed_work(&pwm_work, msecs_to_jiffies(300));
}

static struct tpu_pwm_platform_data pwm_device_data = {
	.init = pwm_init,
	.is_low_at_disable = 1,
};

static struct platform_device pwm_device = {
	.name = "tpu_pwm",
	.id = 2,
	.dev	= {
		.platform_data = &pwm_device_data,
	},
	.num_resources = ARRAY_SIZE(pwm_resources),
	.resource = pwm_resources,
};

static struct platform_pwm_backlight_data pwm_backlight_data = {
	.pwm_id = 2,
	.max_brightness = 255,
	.dft_brightness = 255,
	.pwm_period_ns = 33333, /* 30kHz */
};

static struct platform_device pwm_backlight_device = {
	.name = "pwm-backlight",
	.dev = {
		.platform_data = &pwm_backlight_data,
	},
};

static struct fb_videomode lcd0_modedb[] = {
	[0] = {
		.name		= "AMPIER/AM-800480",
		.refresh	= 55,
		.xres		= 800,
		.yres		= 480,
		.pixclock	= 33670,
		.left_margin	= 88,
		.right_margin	= 40,
		.hsync_len	= 128,
		.upper_margin	= 20,
		.lower_margin	= 5,
		.vsync_len	= 5,
		.sync		= 0,
		.vmode		= FB_VMODE_NONINTERLACED,
		.flag		= 0,
	},
};
static struct sh_videomode_ext lcd0_modedb_ext[] = {
	[0] = {
		.flags = 0,
	},
};
static struct sh_mobile_lcdc_info lcd0_info = {
	.clock_source	= LCDC_CLK_BUS,
	.max_width	= 800,
	.max_height	= 480,

	.ch[0] = {
		.chan = LCDC_CHAN_MAINLCD,
		.bpp = 32,
		.interface_type		= RGB24,
		.clock_divider		= 5,
		.flags			= 0,
		.lcd_cfg = {
			.name		= "WVGA Panel",
			.xres		= 800,
			.yres		= 480,
			.left_margin	= 88,
			.right_margin	= 40,
			.hsync_len	= 128,
			.upper_margin	= 20,
			.lower_margin	= 5,
			.vsync_len	= 5,
			.sync		= 0,
		},
		.lcd_size_cfg = {
			.width	= 111,
			.height = 68,
		},

		.modedb = lcd0_modedb,
		.modedb_ext = lcd0_modedb_ext,
		.modedb_size = ARRAY_SIZE(lcd0_modedb),
		.default_mode = "AMPIER/AM-800480,XRGB8888",
	},
};

static struct resource lcd0_resources[] = {
	[0] = {
		.name	= "LCD0",
		.start	= 0xfe940000,
		.end	= 0xfe943fff,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(177),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device lcd0_device = {
	.name		= "sh_mobile_lcdc_fb",
	.num_resources	= ARRAY_SIZE(lcd0_resources),
	.resource	= lcd0_resources,
	.id = 0,
	.dev	= {
		.platform_data	= &lcd0_info,
		.coherent_dma_mask = ~0,
	},
};

static struct fb_videomode lcd1_modedb[] = {
	[0] = {
		.name		= "1920x1080i@60",
		.refresh	= 60,
		.xres		= 1920,
		.yres		= 1080,
		.pixclock	= 13468,
		.left_margin	= 148,
		.right_margin	= 88,
		.upper_margin	= 30,
		.lower_margin	= 5,
		.hsync_len	= 44,
		.vsync_len	= 10,
		.sync		= FB_SYNC_VERT_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT,
		.vmode		= FB_VMODE_INTERLACED,
		.flag		= 0,
	},
	[1] = {
		.name		= "1920x1080@60",
		.refresh	= 60,
		.xres		= 1920,
		.yres		= 1080,
		.pixclock	= 6734,
		.left_margin	= 148,
		.right_margin	= 88,
		.upper_margin	= 36,
		.lower_margin	= 4,
		.hsync_len	= 44,
		.vsync_len	= 5,
		.sync		= FB_SYNC_VERT_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT,
		.vmode		= FB_VMODE_NONINTERLACED,
		.flag		= 0,
	},
	[2] = {
		.name		= "1920x1080@24",
		.refresh	= 24,
		.xres		= 1920,
		.yres		= 1080,
		.pixclock	= 13468,
		.left_margin	= 148,
		.right_margin	= 638,
		.upper_margin	= 36,
		.lower_margin	= 4,
		.hsync_len	= 44,
		.vsync_len	= 5,
		.sync		= FB_SYNC_VERT_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT,
		.vmode		= FB_VMODE_NONINTERLACED,
		.flag		= 0,
	},
	[3] = {
		.name		= "1280x720@60",
		.refresh	= 60,
		.xres		= 1280,
		.yres		= 720,
		.pixclock	= 13468,
		.left_margin	= 220,
		.right_margin	= 110,
		.upper_margin	= 20,
		.lower_margin	= 5,
		.hsync_len	= 40,
		.vsync_len	= 5,
		.sync		= FB_SYNC_VERT_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT,
		.vmode		= FB_VMODE_NONINTERLACED,
		.flag		= 0,
	},
	[4] = {
		.name		= "720x480@60",
		.refresh	= 60,
		.xres		= 720,
		.yres		= 480,
		.pixclock	= 37037,
		.left_margin	= 60,
		.right_margin	= 16,
		.upper_margin	= 30,
		.lower_margin	= 9,
		.hsync_len	= 62,
		.vsync_len	= 6,
		.sync		= 0,
		.vmode		= FB_VMODE_NONINTERLACED,
		.flag		= 0,
	},
	[5] = {
		.name		= "NTSC",
		.refresh	= 60,
		.xres		= 720,
		.yres		= 480,
		.pixclock	= 37037,
		.left_margin	= 60,
		.right_margin	= 16,
		.upper_margin	= 30,
		.lower_margin	= 9,
		.hsync_len	= 62,
		.vsync_len	= 6,
		.sync		= 0,
		.vmode		= FB_VMODE_INTERLACED,
		.flag		= 0,
	},
};
static struct sh_videomode_ext lcd1_modedb_ext[] = {
	[0] = {
		.flags = 0,
	},
	[1] = {
		.flags = 0,
	},
	[2] = {
		.flags = 0,
	},
	[3] = {
		.flags = 0,
	},
	[4] = {
		.flags = 0,
	},
	[5] = {
		.flags = (LCDC_FLAGS_DWPOL | EXT_FLAG_OUTPUT_YUV422),
	},
};
static struct sh_mobile_lcdc_info lcd1_info = {
	.clock_source	= LCDC_CLK_HDMI,
	.max_width	= 1920,
	.max_height	= 1080,
	.hdmi_dev_name	= "sh_mobile_hdmi.0",

	.ch[0] = {
		.chan = LCDC_CHAN_MAINLCD,
		.bpp = 32,
		.interface_type		= RGB24,
		.clock_divider		= 3,
		.flags			= 0,
		.lcd_cfg = {
			.name		= "1920x1080i@60",
			.refresh	= 60,
			.xres		= 1920,
			.yres		= 1080,
			.pixclock	= 13468,
			.left_margin	= 148,
			.right_margin	= 88,
			.upper_margin	= 15,
			.lower_margin	= 2,
			.hsync_len	= 44,
			.vsync_len	= 5,
			.sync		= (FB_SYNC_VERT_HIGH_ACT |
					   FB_SYNC_HOR_HIGH_ACT)
		},
		.lcd_size_cfg = {
			.width	= 1920,
			.height = 1080,
		},

		.modedb = lcd1_modedb,
		.modedb_ext = lcd1_modedb_ext,
		.modedb_size = ARRAY_SIZE(lcd1_modedb),
		.default_mode = "1920x1080i@60,XRGB8888",
	},
};

static struct resource lcd1_resources[] = {
	[0] = {
		.name	= "LCD1",
		.start	= 0xfe944000,
		.end	= 0xfe947fff,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(178),
		.flags	= IORESOURCE_IRQ,
	},
};

static struct platform_device lcd1_device = {
	.name		= "sh_mobile_lcdc_fb",
	.num_resources	= ARRAY_SIZE(lcd1_resources),
	.resource	= lcd1_resources,
	.id = 1,
	.dev	= {
		.platform_data	= &lcd1_info,
		.coherent_dma_mask = ~0,
	},
};

/* SPI.MSIOF */
static struct spi_board_info __maybe_unused msiof1_board_info[] __initdata = {
	/* this is inplementation dependent. */
	{
		.modalias		= "spidev",
		.controller_data	= (void *)GPIO_PORT120,
		.max_speed_hz		= 1000000,
		.bus_num		= 1,
		.chip_select		= 0,
	}
};

static struct sh_msiof_spi_info __maybe_unused sh_msiof1_info = {
	.rx_fifo_override	= 256,
	.num_chipselect		= 3,
};

static struct resource __maybe_unused sh_msiof1_resources[] = {
	[0] = {
		.start		= 0xE6E10000,
		.end		= 0xE6E10064 - 1,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= gic_spi(91),
		.flags		= IORESOURCE_IRQ,
	},
};

static struct platform_device __maybe_unused sh_msiof1_device = {
	.name		= "spi_sh_msiof",
	.id		= 1,
	.dev		= {
		.platform_data	= &sh_msiof1_info,
	},
	.resource	= sh_msiof1_resources,
	.num_resources	= ARRAY_SIZE(sh_msiof1_resources),
};

static struct gpio __maybe_unused msiof1_gpios[] = {
	{ GPIO_FN_PORT121_MSIOF1_TSCK,	0,	"TSCK" },
	{ GPIO_FN_PORT119_MSIOF1_TXD,	0,	"MISO" },
	{ GPIO_FN_PORT118_MSIOF1_RXD,	0,	"MOSI" },
};

static int __maybe_unused gpio_request_array_simple(const struct gpio *array,
						    size_t num)
{
	int i, err;

	for (i = 0; i < num; i++, array++) {
		err = gpio_request(array->gpio, array->label);
		if (err)
			goto err_free;
	}
	return 0;

err_free:
	while (i--)
		gpio_free((--array)->gpio);
	return err;
}

static void a800_msiof_init(void)
{
#ifndef CONFIG_SPI_SH_MSIOF
//	if (!IS_ENABLED(CONFIG_SPI_SH_MSIOF))
		return;
#endif
	if (gpio_request_array_simple(msiof1_gpios, ARRAY_SIZE(msiof1_gpios))) {
		pr_err("Any MSIOF1 resources are already reserved.\n");
		return;
	}

	platform_device_register(&sh_msiof1_device);

#ifdef CONFIG_SPI_SH_MSIOF
//	if (IS_ENABLED(CONFIG_SPI_SH_MSIOF)) {
		gpio_request(GPIO_PORT120, NULL); /* CS */
		gpio_direction_output(GPIO_PORT120, 0);

		spi_register_board_info(msiof1_board_info,
					ARRAY_SIZE(msiof1_board_info));
//	}
#endif
}

/* I2C devices */
#include <linux/i2c/at24.h>
static struct at24_platform_data at24 = {
	.byte_len = 128,
	.page_size = 16,
};

static struct i2c_board_info i2c0_devices[] = {
	{
		I2C_BOARD_INFO("st1232-ts", 0x55),
		.irq = irqpin2irq(10), /* FMSICK */
	},
	{
		I2C_BOARD_INFO("at24", 0x50),
		.platform_data = &at24,
	},
	{
		I2C_BOARD_INFO("wm8978", 0x1a),
	},
};

static struct i2c_board_info i2c1_devices[] = {
};

static struct i2c_board_info __initdata i2c2_devices[] = {
	{
		I2C_BOARD_INFO("s35390a", 0x30),
		.type ="s35390a",
	},
};

#include <linux/i2c-gpio.h>
static struct i2c_gpio_platform_data gpio_i2c_data = {
	.sda_pin	= GPIO_PORT208,
	.scl_pin	= GPIO_PORT91,
	.udelay = 5,
};

static struct platform_device gpio_i2c = {
	.name = "i2c-gpio",
	.id = 2,
	.dev = {
		.platform_data = &gpio_i2c_data,
	},
};

static struct i2c_board_info i2c_cameras[] = {
	{
		I2C_BOARD_INFO("mt9t112", 0x3d),
	},
	{
		I2C_BOARD_INFO("mt9m113", 0x3c),
	},
	{
		I2C_BOARD_INFO("imx036", 0x60),
	},
};

#if defined(CONFIG_MACH_ARMADILLO800EVA_CEU_MT9T111)
#include <media/mt9t112.h>
static struct mt9t112_camera_info mt9t111_info = {
	.flags = (MT9T112_FLAG_PCLK_RISING_EDGE |
		  MT9T112_FLAG_DATAWIDTH_8 |
		  MT9T112_FLAG_ENABLE_GPIO_FLASH),
	/* EXTCLK=24M, VCO-Output=768M, PIXCLK=96M */
	.divider = { 16, 0, 0, 7, 0, 10, 14, 7, 7 },
};

static struct soc_camera_link outcamera_link = {
	.bus_id		= 0,
	.module_name	= "mt9t112",
	.i2c_adapter_id	= 0,
	.board_info	= &i2c_cameras[0],
	.priv		= &mt9t111_info,
};
#endif

#if defined(CONFIG_MACH_ARMADILLO800EVA_CEU_IMX036)
static struct soc_camera_link outcamera_link = {
	.bus_id		= 0,
	.module_name	= "imx036",
	.i2c_adapter_id	= 0,
	.board_info	= &i2c_cameras[2],
};
#endif

static struct platform_device outcamera_device = {
	.name   = "soc-camera-pdrv",
	.id     = 0,
	.dev    = {
		.platform_data = &outcamera_link,
	},
};

#if defined(CONFIG_MACH_ARMADILLO800EVA_CEU_8BIT)
#define CEU0_INFO_FLAGS SH_CEU_FLAG_USE_8BIT_BUS | SH_CEU_FLAG_USE_LOWER_8BIT_BUS
#else
#define CEU0_INFO_FLAGS SH_CEU_FLAG_USE_16BIT_BUS
#endif

static struct sh_mobile_ceu_info sh_mobile_ceu0_info = {
	.flags = CEU0_INFO_FLAGS,
	.csi2_dev = NULL,
};

static struct resource ceu0_resources[] = {
	[0] = {
		.name	= "CEU0",
		.start	= 0xfe910000,
		.end	= 0xfe9100ab,
		.flags	= IORESOURCE_MEM,
	},
	[1] = {
		.start	= gic_spi(160),
		.flags  = IORESOURCE_IRQ,
	},
	[2] = {
		/* place holder for contiguous memory */
	},
};

static struct platform_device ceu0_device = {
	.name		= "sh_mobile_ceu",
	.id             = 0, /* "ceu0" clock */
	.num_resources	= ARRAY_SIZE(ceu0_resources),
	.resource	= ceu0_resources,
	.dev	= {
		.platform_data	= &sh_mobile_ceu0_info,
		.coherent_dma_mask = DMA_BIT_MASK(32),
	},
};

#include <linux/gpio_keys.h>
static struct gpio_keys_button gpio_key_buttos[] = {
	{KEY_POWER, GPIO_PORT99, 1, "SW1", EV_KEY, 1, 100, 1},
	{KEY_BACK, GPIO_PORT100, 1, "SW2", EV_KEY, 1, 100, 1},
	{KEY_MENU, GPIO_PORT97, 1, "SW3", EV_KEY, 1, 100, 1},
	{KEY_HOME, GPIO_PORT98, 1, "SW4", EV_KEY, 1, 100, 1},
};

static struct gpio_keys_platform_data gpio_key_data = {
	.buttons = gpio_key_buttos,
	.nbuttons = ARRAY_SIZE(gpio_key_buttos),
};

static struct platform_device gpio_key = {
	.name = "gpio-keys",
	.id   = 0,
	.dev = {
		.platform_data = &gpio_key_data,
	},
};

#define GPIO_LED(_name, _default_trigger, _gpio, _active_low)	\
{								\
	.name = _name,						\
	.default_trigger = _default_trigger,			\
	.gpio = _gpio,						\
	.active_low = _active_low,				\
}

static struct gpio_led gpio_led_pins[] = {
	GPIO_LED("LED3", "none", GPIO_PORT102, 0),
	GPIO_LED("LED4", "none", GPIO_PORT111, 0),
	GPIO_LED("LED5", "none", GPIO_PORT110, 0),
	GPIO_LED("LED6", "none", GPIO_PORT177, 0),
};

static struct gpio_led_platform_data gpio_led_data = {
	.leds = gpio_led_pins,
	.num_leds = ARRAY_SIZE(gpio_led_pins),
};

static struct platform_device gpio_led = {
	.name = "leds-gpio",
	.id = 0,
	.dev = {
		.platform_data = &gpio_led_data,
	},
};

static struct platform_device *rma1evb_devices[] __initdata = {
	&eth_device,
	&sh_mmcif_device,
	&sdhi0_device,
	&sdhi1_device,
	&ohci_device,
	&ehci_device,
	&hdmi_device,
	&sdenc_device,
#if defined(CONFIG_MACH_ARMADILLO800EVA_PRIMARY_FB_LCDC0)
	&lcd0_device,
	&lcd1_device,
#else
	&lcd1_device,
	&lcd0_device,
#endif
	&pwm_device,
	&pwm_backlight_device,
	&fsi_device,
	&fsi_hdmi_codec_device,
	&ceu0_device,
	&outcamera_device,
	&gpio_i2c,
	&gpio_key,
	&gpio_led,
};

static struct map_desc rma1evb_io_desc[] __initdata = {
	/* 0xe6000000-0xf0ffffff -> 0xe6000000-0xf0ffffff (Peri) */
	{
		.virtual	= 0xe6000000,
		.pfn		= __phys_to_pfn(0xe6000000),
		.length		= 0x0b000000,
		.type		= MT_DEVICE_NONSHARED
	},
	/* 0xc1000000-0xc7ffffff -> 0xf1000000-0xf7ffffff (AXI Peri) */
	{
		.virtual	= 0xf1000000,
		.pfn		= __phys_to_pfn(0xc1000000),
		.length		= 0x07000000,
		.type		= MT_DEVICE_NONSHARED
	},
};

static void __init rma1evb_map_io(void)
{
	iotable_init(rma1evb_io_desc, ARRAY_SIZE(rma1evb_io_desc));

	/* setup early devices and console here as well */
	sh7740_add_early_devices();
	shmobile_setup_console();
}

void __init rma1evb_init_irq(void)
{
	sh7740_init_irq();
}

#define RESCNT2		0xe6188020

void rma1evb_arch_reset(char mode, const char *cmd)
{
	__raw_writel(__raw_readl(RESCNT2) | (1 << 31), RESCNT2);
}

#define print_clock(id, clk) \
({	if (0) printk("%16s = %ld\n", id, clk_get_rate(clk)); 0; })
#define print_clock_raw(fmt, args...)			\
({	if (0) printk(fmt, ## args); 0; })

static void __init rma1evb_init(void)
{
	struct clk *xtal1 = clk_get(NULL, "extal1");
	struct clk *sub_clk = clk_get(NULL, "sub_clk");
	struct clk *sub2_clk = clk_get(NULL, "sub2_clk");
	struct clk *pllc1_div2 = clk_get(NULL, "pllc1_div2_clk");
	struct clk *fsiack_clk = clk_get(NULL, "fsiack_clk");
	struct clk *fsia_clk = clk_get(NULL, "fsia_clk");
	struct clk *pllc1_clk = clk_get(NULL, "pllc1_clk");
	struct clk *hdmi_clk = clk_get(NULL, "hdmi_clk");
	struct clk *system_clk = clk_get(NULL, "system_clk");
	struct clk *cp_clk = clk_get(NULL, "cp_clk");
	struct clk *dv_clk = clk_get(NULL, "dv_clk");
	struct clk *ceu0_clk;
	struct clk *cko_clk;
	unsigned long rate;

	print_clock_raw("=== System clocks ===\n");
	clk_set_parent(sub_clk, pllc1_div2);
	clk_set_parent(sub2_clk, pllc1_div2);
	rate = clk_round_rate(sub_clk, 50000000);
	clk_set_rate(sub_clk, rate);
	clk_set_rate(sub2_clk, rate);
	clk_put(sub_clk);
	clk_put(sub2_clk);
	clk_put(pllc1_div2);

	shmobile_arch_reset = rma1evb_arch_reset;

	sh7740_pinmux_init();

	/* SCIFA1 */
	gpio_request(GPIO_FN_SCIFA1_RXD, NULL);
	gpio_request(GPIO_FN_SCIFA1_TXD, NULL);

	/* GETHER */
	gpio_request(GPIO_FN_ET_CRS, NULL);
	gpio_request(GPIO_FN_ET_MDC, NULL);
	gpio_request(GPIO_FN_ET_MDIO, NULL);
	gpio_request(GPIO_FN_ET_TX_ER, NULL);
	gpio_request(GPIO_FN_ET_RX_ER, NULL);
	gpio_request(GPIO_FN_ET_ERXD0, NULL);
	gpio_request(GPIO_FN_ET_ERXD1, NULL);
	gpio_request(GPIO_FN_ET_ERXD2, NULL);
	gpio_request(GPIO_FN_ET_ERXD3, NULL);
	gpio_request(GPIO_FN_ET_TX_CLK, NULL);
	gpio_request(GPIO_FN_ET_TX_EN, NULL);
	gpio_request(GPIO_FN_ET_ETXD0, NULL);
	gpio_request(GPIO_FN_ET_ETXD1, NULL);
	gpio_request(GPIO_FN_ET_ETXD2, NULL);
	gpio_request(GPIO_FN_ET_ETXD3, NULL);
	gpio_request(GPIO_FN_ET_PHY_INT, NULL);
	gpio_request(GPIO_FN_ET_COL, NULL);
	gpio_request(GPIO_FN_ET_RX_DV, NULL);
	gpio_request(GPIO_FN_ET_RX_CLK, NULL);

	gpio_request(GPIO_PORT18, NULL); /* PHY_RST */
	gpio_direction_output(GPIO_PORT18, 1);
	mdelay(10);
	gpio_direction_output(GPIO_PORT18, 1);

	/* SDHI0 */
	gpio_request(GPIO_FN_SDHICMD0, NULL);
	gpio_request(GPIO_FN_SDHICLK0, NULL);
	gpio_request(GPIO_FN_SDHID0_0, NULL);
	gpio_request(GPIO_FN_SDHID0_1, NULL);
	gpio_request(GPIO_FN_SDHID0_2, NULL);
	gpio_request(GPIO_FN_SDHID0_3, NULL);
	gpio_request(GPIO_FN_SDHICD0, NULL);
	gpio_request(GPIO_FN_SDHIWP0, NULL);

	gpio_request(GPIO_PORT17, NULL);	/* SDHI0_18/33_B */
	gpio_direction_output(GPIO_PORT17, 0);
	gpio_request(GPIO_PORT74, NULL);	/* SDHI0_PON */
	gpio_direction_output(GPIO_PORT74, 1);
	gpio_request(GPIO_PORT75, NULL);	/* SDSLOT1_PON */
	gpio_direction_output(GPIO_PORT75, 1);
	gpio_request(GPIO_PORT167, NULL);	/* SDHI0_CD */
	gpio_direction_input(GPIO_PORT167);

	/* SDHI1 */
	gpio_request(GPIO_FN_SDHICLK1, NULL);
	gpio_request(GPIO_FN_SDHICMD1, NULL);
	gpio_request(GPIO_FN_SDHID1_0, NULL);
	gpio_request(GPIO_FN_SDHID1_1, NULL);
	gpio_request(GPIO_FN_SDHID1_2, NULL);
	gpio_request(GPIO_FN_SDHID1_3, NULL);
	gpio_request(GPIO_FN_SDHICD1, NULL);
	gpio_request(GPIO_FN_SDHIWP1, NULL);

	gpio_request(GPIO_PORT16, NULL); /* SDSLOT2_PON */
	gpio_direction_output(GPIO_PORT16, 1);

	/* MMCIF */
	gpio_request(GPIO_FN_PORT103_MMCCLK0, NULL);
	gpio_request(GPIO_FN_PORT104_MMCCMD0, NULL);
	gpio_request(GPIO_FN_PORT149_MMCD0_0, NULL);
	gpio_request(GPIO_FN_PORT148_MMCD0_1, NULL);
	gpio_request(GPIO_FN_PORT147_MMCD0_2, NULL);
	gpio_request(GPIO_FN_PORT146_MMCD0_3, NULL);
	gpio_request(GPIO_FN_PORT145_MMCD0_4, NULL);
	gpio_request(GPIO_FN_PORT144_MMCD0_5, NULL);
	gpio_request(GPIO_FN_PORT143_MMCD0_6, NULL);
	gpio_request(GPIO_FN_PORT142_MMCD0_7, NULL);

	gpio_request(GPIO_PORT199, NULL); /* MMC_RST_B */
	gpio_direction_output(GPIO_PORT199, 0);
	mdelay(10);
	gpio_direction_output(GPIO_PORT199, 1);

	/* LCDC0 */
	gpio_request(GPIO_FN_PORT1_LCDD23_0,   NULL);
	gpio_request(GPIO_FN_PORT0_LCDD22_0,   NULL);
	gpio_request(GPIO_FN_PORT2_LCDD21_0,   NULL);
	gpio_request(GPIO_FN_PORT3_LCDD20_0,   NULL);
	gpio_request(GPIO_FN_PORT4_LCDD19_0,   NULL);
	gpio_request(GPIO_FN_PORT40_LCDD18_0,   NULL);
	gpio_request(GPIO_FN_LCDD17_0,   NULL);
	gpio_request(GPIO_FN_LCDD16_0,   NULL);
	gpio_request(GPIO_FN_LCDD15_0,   NULL);
	gpio_request(GPIO_FN_LCDD14_0,   NULL);
	gpio_request(GPIO_FN_LCDD13_0,   NULL);
	gpio_request(GPIO_FN_LCDD12_0,   NULL);
	gpio_request(GPIO_FN_LCDD11_0,   NULL);
	gpio_request(GPIO_FN_LCDD10_0,   NULL);
	gpio_request(GPIO_FN_LCDD9_0,    NULL);
	gpio_request(GPIO_FN_LCDD8_0,    NULL);
	gpio_request(GPIO_FN_LCDD7_0,    NULL);
	gpio_request(GPIO_FN_LCDD6_0,    NULL);
	gpio_request(GPIO_FN_LCDD5_0,    NULL);
	gpio_request(GPIO_FN_LCDD4_0,    NULL);
	gpio_request(GPIO_FN_LCDD3_0,    NULL);
	gpio_request(GPIO_FN_LCDD2_0,    NULL);
	gpio_request(GPIO_FN_LCDD1_0,    NULL);
	gpio_request(GPIO_FN_LCDD0_0,    NULL);
	gpio_request(GPIO_FN_LCDDCK_0,   NULL);
	gpio_request(GPIO_FN_LCDVSYN_0,  NULL);
	gpio_request(GPIO_FN_LCDHSYN_0,  NULL);
	gpio_request(GPIO_FN_LCDDISP_0,  NULL);
	gpio_request(GPIO_FN_LCDRD_0,    NULL);
	gpio_request(GPIO_FN_PORT165_LCDLCLK_0,  NULL);

	gpio_request(GPIO_PORT61, NULL); /* LCDDON */
	gpio_direction_output(GPIO_PORT61, 1);

	gpio_request(GPIO_PORT202, NULL); /* LCD0_LED_CONT */
	gpio_direction_output(GPIO_PORT202, 1);
	gpio_free(GPIO_PORT202);

	gpio_request(GPIO_PORT176, NULL); /* DBGMD_SELECT_B */
	gpio_direction_output(GPIO_PORT176, 1);

	/* HDMI */
	clk_set_parent(hdmi_clk, pllc1_div2);
	clk_set_rate(hdmi_clk, 74250000);
	clk_enable(hdmi_clk);
	print_clock("hdmi_clk", hdmi_clk);
	clk_put(hdmi_clk);

	/* SDENC */
	__raw_writew(0x0001, 0xe6058148);
	__raw_writel(readl(0xe6058024) | 0x80000, 0xe6058024);

	gpio_request(GPIO_FN_LCDDCK_1, NULL);
	gpio_request(GPIO_FN_LCDDON_1, NULL);
	gpio_request(GPIO_FN_LCDHSYN_1, NULL);
	gpio_request(GPIO_FN_LCDVSYN_1, NULL);

	clk_set_rate(dv_clk, 27000000);
	print_clock("dv_clk", dv_clk);
	clk_enable(dv_clk);
	__raw_writel(readl(0xe6150140) & ~0x10000000, 0xe6150140);

	/* FSIA */
	gpio_request(GPIO_FN_FSIAIBT, NULL);
	gpio_request(GPIO_FN_FSIAILR, NULL);
	gpio_request(GPIO_FN_FSIAOMC, NULL);
	gpio_request(GPIO_FN_FSIACK, NULL);
	gpio_request(GPIO_FN_FSIAOSLD, NULL);
	gpio_request(GPIO_FN_PORT5_FSIAISLD, NULL);

	gpio_request(GPIO_PORT7,  NULL);
	gpio_request(GPIO_PORT8, NULL);
#define GPIO_PORT7CR    0xe6050007
#define GPIO_PORT8CR    0xe6050008
	gpio_no_direction(GPIO_PORT7CR); /* FSIAOBT needs no direction */
	gpio_no_direction(GPIO_PORT8CR); /* FSIAOLR needs no direction */

	/* CPU::ACK <- Codec(OSC:12.288MHz) */
	clk_set_parent(fsia_clk, fsiack_clk);
	clk_enable(clk_get(NULL, "fsi"));

	/* USB */
#define USBCKCR		0xe615008c
	writel(readl(USBCKCR) | 0x40, USBCKCR);

	gpio_request(GPIO_FN_PPON_0, NULL);
	gpio_request(GPIO_FN_OCI_0, NULL);
	gpio_request(GPIO_FN_PPON_1, NULL);
	gpio_request(GPIO_FN_OCI_1, NULL);
	usb_clk = clk_get(NULL, "usb_clk");
	usbp_clk = clk_get(NULL, "usbp_clk");
	usbh_clk = clk_get(NULL, "usbh");
	usbphy_clk = clk_get(NULL, "usb_phy");
	clk_set_parent(usb_clk, xtal1);
	gpio_request(GPIO_PORT209, NULL);
	gpio_direction_input(GPIO_PORT209);

	gpio_request(GPIO_PORT159, NULL); /* USB_DEVICE_MODE */
	gpio_direction_input(GPIO_PORT159);

	if (gpio_get_value(GPIO_PORT159)) {
		/* USB Host */
	} else {
		/* USB Func */
		platform_device_register(&usb_func_device);
	}

	clk_enable(usb_clk);

	/* CEU0 */
#if defined(CONFIG_MACH_ARMADILLO800EVA_CEU_16BIT)
	gpio_request(GPIO_FN_PORT24_VIO_D15_0, NULL);
	gpio_request(GPIO_FN_PORT25_VIO_D14_0, NULL);
	gpio_request(GPIO_FN_PORT26_VIO_D13_0, NULL);
	gpio_request(GPIO_FN_VIO_D12_0, NULL);
	gpio_request(GPIO_FN_VIO_D11_0, NULL);
	gpio_request(GPIO_FN_VIO_D10_0, NULL);
	gpio_request(GPIO_FN_VIO_D9_0, NULL);
	gpio_request(GPIO_FN_VIO_D8_0, NULL);
#endif
	gpio_request(GPIO_FN_VIO_D7_0, NULL);
	gpio_request(GPIO_FN_VIO_D6_0, NULL);
	gpio_request(GPIO_FN_VIO_D5_0, NULL);
	gpio_request(GPIO_FN_VIO_D4_0, NULL);
	gpio_request(GPIO_FN_VIO_D3_0, NULL);
	gpio_request(GPIO_FN_VIO_D2_0, NULL);
	gpio_request(GPIO_FN_VIO_D1_0, NULL);
	gpio_request(GPIO_FN_VIO_D0_0, NULL);
	gpio_request(GPIO_FN_VIO_CLK_0, NULL);
	gpio_request(GPIO_FN_VIO_CKO, NULL);
	gpio_request(GPIO_FN_VIO_HD_0, NULL);
	gpio_request(GPIO_FN_VIO_VD_0, NULL);
	gpio_request(GPIO_FN_VIO_FIELD_0, NULL);

	gpio_request(GPIO_PORT173, NULL); /* STANDBY */
	gpio_direction_output(GPIO_PORT173, 0);
	gpio_request(GPIO_PORT172, NULL); /* RST_B */
	gpio_direction_output(GPIO_PORT172, 0);
	mdelay(10);
	gpio_request(GPIO_PORT158, NULL); /* CAM_PON */
	gpio_direction_output(GPIO_PORT158, 1);

	ceu0_clk = clk_get(NULL, "ceu0");
	if (!IS_ERR(ceu0_clk)) {
		clk_enable(ceu0_clk);
		clk_put(ceu0_clk);
	}

	cko_clk = clk_get(NULL, "vck1_clk");
	print_clock("vck1", cko_clk);
	clk_set_rate(cko_clk, 23760000);
	clk_enable(cko_clk);
	clk_put(cko_clk);

	mdelay(50);
	gpio_direction_output(GPIO_PORT172, 1); /* RESET OFF */
	mdelay(100);

	print_clock("extal1", xtal1);
	print_clock("pllc1_clk", pllc1_clk);
	print_clock("sub_clk", sub_clk);
	print_clock("sub2_clk", sub2_clk);
	print_clock("pllc1_div2", pllc1_div2);
	print_clock("fsiack_clk", fsiack_clk);
	print_clock("fsia_clk", fsia_clk);
	print_clock("usb_clk", usb_clk);
	print_clock("usbp_clk", usbp_clk);
	print_clock("usbh_clk", usbh_clk);
	print_clock("usbphy_clk", usbphy_clk);
	print_clock("ceu0_clk", ceu0_clk);
	print_clock("cko_clk", cko_clk);
	print_clock("system_clk", system_clk);
	print_clock("cp_clk", cp_clk);
	print_clock_raw("=====================\n");

	/* a800eva ts-int */
	gpio_request(GPIO_PORT19, NULL);
	gpio_direction_input(GPIO_PORT19);

	/* a800eva ts-rstb */
	gpio_request(GPIO_PORT166, NULL);
	gpio_direction_output(GPIO_PORT166, 1);

#ifdef CONFIG_CACHE_L2X0
	/* Early BRESP enable, Shared attribute override enable, 32K*8way */
	l2x0_init(__io(0xf0100000), 0x40440000, 0x82000fff);
#endif

	i2c_register_board_info(0, i2c0_devices, ARRAY_SIZE(i2c0_devices));
	i2c_register_board_info(1, i2c1_devices, ARRAY_SIZE(i2c1_devices));
	i2c_register_board_info(2, i2c2_devices, ARRAY_SIZE(i2c2_devices));

	a800_msiof_init();

	sh7740_add_standard_devices();

	platform_add_devices(rma1evb_devices, ARRAY_SIZE(rma1evb_devices));
}

static void __init armadillo800eva_timer_init(void)
{
	struct clk *xtal1;

	sh7740_clock_init(0, 0, 1);

	xtal1 = clk_get(NULL, "extal1");
	clk_set_rate(xtal1, 24000000);
	shmobile_timer.init();
}

struct sys_timer rma1evb_timer = {
	.init	= armadillo800eva_timer_init,
};

MACHINE_START(ARMADILLO800EVA, "Armadillo-800EVA")
	.phys_io	= 0xe8080000,
	.io_pg_offst	= ((0xe8080000) >> 18) & 0xfffc,
	.boot_params    = PHYS_OFFSET + 0x100,
	.map_io		= rma1evb_map_io,
	.init_irq	= rma1evb_init_irq,
	.init_machine	= rma1evb_init,
	.timer		= &rma1evb_timer,
MACHINE_END
